==> bifrost-0.1.0-20150902.191116-2.jar: bifrost/core.clj <==
(ns bifrost.core
  (:require [io.pedestal.interceptor :refer [IntoInterceptor interceptor]]
            [clojure.core.async :as async]
            [ring.util.response :as ring-resp]))

(def ^:dynamic *response-timeout* 10000)

(def timeout-response
  {:status :error
   :error {:type :timeout
           :duration *response-timeout*}})

(defn error-response->http-status
  [error-response]
  (case (:type error-response)
    :semantic 400
    :validation 400
    :not-found 404
    :server 500
    :timeout 504
    500))

(defn response->http-status
  [response]
  (case (:status response)
    :ok 200
    :created 201
    :error (error-response->http-status (:error response))
    500))

(defn params-map
  [request]
  (case (:request-method request)
    (:get :delete) (merge (:query-params request)
                          (:path-params request)
                          (:bifrost-params request))
    (:post :put :patch) (merge (:query-params request)
                               (:form-params request)
                               (:edn-params request)
                               (:transit-params request)
                               (:json-params request)
                               (:body-params request)
                               (:path-params request)
                               (:bifrost-params request))))

(defn update-interceptor
  [type source-key-path target-key-path f & args]
  (interceptor
   (let [context-key (case type
                       :enter :request
                       :leave :response)]
     {type
      (fn [ctx]
        (let [old-value (get-in ctx (cons context-key source-key-path))
              new-value (apply f old-value args)
              new-ctx (assoc-in ctx (cons context-key target-key-path) new-value)]
          (if (= source-key-path target-key-path)
            new-ctx
            (update-in new-ctx (butlast source-key-path)
                       dissoc (last source-key-path)))))})))

(defn update-in-request
  ([key-path f] (update-in-request key-path key-path f))
  ([source-key-path target-key-path f & args]
   (apply update-interceptor :enter source-key-path target-key-path f args)))

(defn update-in-response
  ([key-path f] (update-in-response key-path key-path f))
  ([source-key-path target-key-path f & args]
   (apply update-interceptor :leave source-key-path target-key-path f args)))

(defn channel->interceptor
  [channel]
  (interceptor
   {:enter
    (fn [ctx]
      (let [response-channel (async/chan 1)
            request (:request ctx)]
        (async/>!! channel [response-channel (params-map request)])
        (async/go
          (let [response (async/alt! (async/timeout *response-timeout*) timeout-response
                                     response-channel ([r] r))
                status (response->http-status response)]
            (assoc ctx :response
                   (-> response
                       (dissoc :status)
                       ring-resp/response
                       (ring-resp/status status)))))))}))

(extend-protocol IntoInterceptor
  clojure.core.async.impl.channels.ManyToManyChannel
  (-interceptor [t] (channel->interceptor t)))

==> bifrost-0.1.0-20150902.192848-3.jar: bifrost/core.clj <==
(ns bifrost.core
  (:require [io.pedestal.interceptor :refer [IntoInterceptor interceptor]]
            [clojure.core.async :as async]
            [ring.util.response :as ring-resp]))

(def ^:dynamic *response-timeout* 10000)

(def timeout-response
  {:status :error
   :error {:type :timeout
           :duration *response-timeout*}})

(defn error-response->http-status
  [error-response]
  (case (:type error-response)
    :semantic 400
    :validation 400
    :not-found 404
    :server 500
    :timeout 504
    500))

(defn response->http-status
  [response]
  (case (:status response)
    :ok 200
    :created 201
    :error (error-response->http-status (:error response))
    500))

(defn params-map
  [request]
  (case (:request-method request)
    (:get :delete) (merge (:query-params request)
                          (:path-params request)
                          (:bifrost-params request))
    (:post :put :patch) (merge (:query-params request)
                               (:form-params request)
                               (:edn-params request)
                               (:transit-params request)
                               (:json-params request)
                               (:body-params request)
                               (:path-params request)
                               (:bifrost-params request))))

(defn update-interceptor
  [type source-key-path target-key-path f & args]
  (interceptor
   (let [context-key (case type
                       :enter :request
                       :leave :response)]
     {type
      (fn [ctx]
        (let [source-context-path (cons context-key source-key-path)
              target-context-path (cons context-key target-key-path)
              old-value (get-in ctx source-context-path)
              new-value (apply f old-value args)
              new-ctx (assoc-in ctx target-context-path new-value)]
          (if (= source-context-path target-context-path)
            new-ctx
            (update-in new-ctx (butlast source-context-path)
                       dissoc (last source-context-path)))))})))

(defn update-in-request
  ([key-path f] (update-in-request key-path key-path f))
  ([source-key-path target-key-path f & args]
   (apply update-interceptor :enter source-key-path target-key-path f args)))

(defn update-in-response
  ([key-path f] (update-in-response key-path key-path f))
  ([source-key-path target-key-path f & args]
   (apply update-interceptor :leave source-key-path target-key-path f args)))

(defn channel->interceptor
  [channel]
  (interceptor
   {:enter
    (fn [ctx]
      (let [response-channel (async/chan 1)
            request (:request ctx)]
        (async/>!! channel [response-channel (params-map request)])
        (async/go
          (let [response (async/alt! (async/timeout *response-timeout*) timeout-response
                                     response-channel ([r] r))
                status (response->http-status response)]
            (assoc ctx :response
                   (-> response
                       (dissoc :status)
                       ring-resp/response
                       (ring-resp/status status)))))))}))

(extend-protocol IntoInterceptor
  clojure.core.async.impl.channels.ManyToManyChannel
  (-interceptor [t] (channel->interceptor t)))
